{% extends "labeler_base.html" %}
{% block title %}Reset Password{% endblock %}
{% block content %}
<style>
  .container {
    max-width: 500px;
    margin: 50px auto;
    padding: 30px;
    background-color: #f8f9fa;
    border: 1px solid #dee2e6;
    border-radius: 8px;
    box-shadow: 0 2px 10px rgba(0, 0, 0, 0.1);
  }
  h2 {
    text-align: center;
    margin-bottom: 30px;
    color: #343a40;
  }
  .form-group {
    margin-bottom: 20px;
  }
  .form-group label {
    font-weight: bold;
    color: #495057;
  }
  .form-control {
    border-radius: 4px;
    border: 1px solid #ced4da;
    padding: 10px;
  }
  .btn-primary {
    width: 100%;
    padding: 10px;
    font-size: 16px;
    background-color: #007bff;
    border-color: #007bff;
    border-radius: 4px;
  }
  .btn-primary:hover {
    background-color: #0056b3;
    border-color: #004085;
  }
  p {
    text-align: center;
    margin-top: 20px;
    color: #6c757d;
  }
  p a {
    color: #007bff;
    text-decoration: none;
  }
  p a:hover {
    text-decoration: underline;
  }
</style>
<div class="container">
    <h2>Reset Password</h2>
    <form method="POST" action="{{ url_for('auth.reset_password') }}">
        <div class="form-group">
            <label for="identifier">Username or Email</label>
            <input type="text" id="identifier" name="identifier" class="form-control" required>
        </div>
        <div class="form-group">
            <label for="password">New Password</label>
            <input type="password" id="password" name="password" class="form-control" required>
        </div>
        <div class="form-group">
            <label for="password_confirm">Confirm New Password</label>
            <input type="password" id="password_confirm" name="password_confirm" class="form-control" required>
        </div>
        <button type="submit" class="btn btn-primary">Reset</button>
    </form>
    <p>Remember your password? <a href="{{ url_for('auth.login') }}">Login</a></p>
</div>
{% endblock %}